/*
  PT2257 Volume controler
  FBE=0xFC; FBH=0xD7; FBL=0xF7

*/
#include <IRremote.h>
#include <EEPROM.h>
#include <Wire.h>

//definicja przyporzadkowan klawiszy
#define SW_VUP A3  // Vol_UP PC3
#define SW_VDN A2  // Vol_UP PC2

//definicja przyporzadkowan kodow RC5
#define IR_VUP            0x036
#define IR_VDN            0x037

#define PT2257_Adr 0x44

void key_read(void);
void led_bar(void);
void ee_store(void);
void PT2257_write(void);

int SWDelay = 200;

int VOL; // Vol level
int VOLH, VOLL; // PT2257 Hig/Low Reg Volume

int RECV_PIN = 11;
int RC5_FLAG = 0;
IRrecv irrecv(RECV_PIN);
decode_results results;

int L11 = 4; // Vol_Led 11 max
int L10 = 3; // Vol_Led 10 
int L9 = 2; // Vol_Led 9 
int L8 = 1; // Vol_Led 8
int L7 = 0; // Vol_Led 7
int L6 = A1; // Vol_Led 6
int L5 = A0; // Vol_Led 5
int L4 = 8; // Vol_Led 4
int L3 = 7; // Vol_Led 3
int L2 = 6; // Vol_Led 2
int L1 = 5; // Vol_Led 1 min

int LON = 9; // Led_ON 

void setup() {
  Wire.begin();
  VOL = EEPROM.read(1);
  if (VOL > 32) VOL = 32;

  pinMode(SW_VUP, INPUT);
  digitalWrite(SW_VUP, HIGH);
  pinMode(SW_VDN, INPUT);
  digitalWrite(SW_VDN, HIGH);

  pinMode(LON, OUTPUT);
  pinMode(L11, OUTPUT); 
  pinMode(L10, OUTPUT);
  pinMode(L9, OUTPUT);
  pinMode(L8, OUTPUT);
  pinMode(L7, OUTPUT);
  pinMode(L6, OUTPUT);
  pinMode(L5, OUTPUT);
  pinMode(L4, OUTPUT);
  pinMode(L3, OUTPUT);
  pinMode(L2, OUTPUT);
  pinMode(L1, OUTPUT);
 
  led_bar();
  PT2257_write();
  irrecv.enableIRIn(); // Start the receiver
}

void led_bar()
{
  digitalWrite(L11, HIGH);
  digitalWrite(L10, HIGH);
  digitalWrite(L9, HIGH);
  digitalWrite(L8, HIGH);
  digitalWrite(L7, HIGH);
  digitalWrite(L6, HIGH);
  digitalWrite(L5, HIGH);
  digitalWrite(L4, HIGH);
  digitalWrite(L3, HIGH);
  digitalWrite(L2, HIGH);
  digitalWrite(L1, HIGH);
  digitalWrite(LON, HIGH);

  switch (VOL) {
    case 0: {
        digitalWrite(L1, LOW);
        VOLH = 0xE7; VOLL = 0xD6; //-76dB
        break;
      };
    case 1: {
        digitalWrite(L1, LOW);
        VOLH = 0xE7; VOLL = 0xD0; //-70dB
        break;
      };
    case 2: {
        digitalWrite(L1, LOW);
        VOLH = 0xE6; VOLL = 0xD4; //-64dB
        break;
      };
    case 3: {
        digitalWrite(L2, LOW);
        VOLH = 0xE5; VOLL = 0xD8; //-58dB
        break;
      };
    case 4: {
        digitalWrite(L2, LOW);
        VOLH = 0xE5; VOLL = 0xD6; //-56dB
        break;
      };
    case 5: {
        digitalWrite(L2, LOW);
        VOLH = 0xE5; VOLL = 0xD4; //-54dB
        break;
      };
    case 6: {
        digitalWrite(L3, LOW);
        VOLH = 0xE5; VOLL = 0xD2; //-52dB
        break;
      };
    case 7: {
        digitalWrite(L3, LOW);
        VOLH = 0xE5; VOLL = 0xD0; //-50dB
        break;
      };
    case 8: {
        digitalWrite(L3, LOW);
        VOLH = 0xE4; VOLL = 0xD8; //-48dB
        break;
      };
    case 9: {
        digitalWrite(L4, LOW);
        VOLH = 0xE4; VOLL = 0xD6; //-46dB
        break;
      };
    case 10: {
        digitalWrite(L4, LOW);
        VOLH = 0xE4; VOLL = 0xD4; //-44dB
        break;
      };
    case 11: {
        digitalWrite(L4, LOW);
        VOLH = 0xE4; VOLL = 0xD2; //-42dB
        break;
      };
    case 12: {
        digitalWrite(L5, LOW);
        VOLH = 0xE4; VOLL = 0xD0; //-40dB
        break;
      };
    case 13: {
        digitalWrite(L5, LOW);
        VOLH = 0xE3; VOLL = 0xD8; //-38dB
        break;
      };
    case 14: {
        digitalWrite(L5, LOW);
        VOLH = 0xE3; VOLL = 0xD6; //-36dB
        break;
      };
    case 15: {
        digitalWrite(L6, LOW);
        VOLH = 0xE3; VOLL = 0xD4; //-34dB
        break;
      };
    case 16: {
        digitalWrite(L6, LOW);
        VOLH = 0xE3; VOLL = 0xD2; //-32dB
        break;
      };
    case 17: {
        digitalWrite(L6, LOW);
        VOLH = 0xE3; VOLL = 0xD0; //-30dB
        break;
      };
    case 18: {
        digitalWrite(L7, LOW);
        VOLH = 0xE2; VOLL = 0xD8; //-28dB
        break;
      };
    case 19: {
        digitalWrite(L7, LOW);
        VOLH = 0xE2; VOLL = 0xD6; //-26dB
        break;
      };
    case 20: {
        digitalWrite(L7, LOW);
        VOLH = 0xE2; VOLL = 0xD4; //-24dB
        break;
      };
    case 21: {
        digitalWrite(L8, LOW);
        VOLH = 0xE2; VOLL = 0xD2; //-22dB
        break;
      };
    case 22: {
        digitalWrite(L8, LOW);
        VOLH = 0xE2; VOLL = 0xD0; //-20dB
        break;
      };
    case 23: {
        digitalWrite(L8, LOW);
        VOLH = 0xE1; VOLL = 0xD8; //-18dB
        break;
      };
    case 24: {
        digitalWrite(L9, LOW);
        VOLH = 0xE1; VOLL = 0xD6; //-16dB
        break;
      };
    case 25: {
        digitalWrite(L9, LOW);
        VOLH = 0xE1; VOLL = 0xD4; //-14dB
        break;
      };
    case 26: {
        digitalWrite(L9, LOW);
        VOLH = 0xE1; VOLL = 0xD2; //-12dB
        break;
      };
    case 27: {
        digitalWrite(L10, LOW);
        VOLH = 0xE1; VOLL = 0xD0; //-10dB
        break;
      };
    case 28: {
        digitalWrite(L10, LOW);
        VOLH = 0xE0; VOLL = 0xD8; //-8dB
        break;
      };
    case 29: {
        digitalWrite(L10, LOW);
        VOLH = 0xE0; VOLL = 0xD6; //-6dB
        break;
      };
    case 30: {
        digitalWrite(L11, LOW);
        VOLH = 0xE0; VOLL = 0xD4; //-4dB
        break;
      };
    case 31: {
        digitalWrite(L11, LOW);
        VOLH = 0xE0; VOLL = 0xD2; //-2dB
        break;
      };
    case 32: {
        digitalWrite(L11, LOW);
        VOLH = 0xE0; VOLL = 0xD0; //0dB
        break;
      };
  }
}

void key_read()
{
  if (digitalRead(SW_VUP) == 0)
  {
    if (VOL < 32) ++VOL;
    else VOL = 32;
    led_bar();
    PT2257_write();
  }
  if (digitalRead(SW_VDN) == 0)
  {
    if (VOL > 0) --VOL;
    else VOL = 0;
    led_bar();
    PT2257_write();
  }
}

void RC5_Check(decode_results *results) {
  if (results->decode_type == RC5)   {
    //        Serial.print("Decoded RC5: ");
    RC5_FLAG = 1;
  }
  else  {
    RC5_FLAG = 0;
  }
}


void ir_receive()
{
  if (irrecv.decode(&results)) {
    results.value = results.value & 0x0FF;
    //        Serial.println(results.value, HEX);
    if (RC5_FLAG == 1) {
      switch (results.value) {
        case IR_VUP: //Vol+
          {
            if (VOL < 32) ++VOL;
            else VOL = 32;
            led_bar();
            PT2257_write();
            break;
          }
        case IR_VDN: //Vol-
          {
            if (VOL > 0) --VOL;
            else VOL = 0;
            led_bar();
            PT2257_write();
            break;
          }
      }
    }
    RC5_Check(&results);
    irrecv.resume();
  }
}

void ee_store()
{
  int EEVOL;
  EEVOL = EEPROM.read(1);
  if (EEVOL > 32) EEVOL = 32;

  if (EEVOL != VOL) {
    EEPROM.write(1, VOL);
    delay(10);
  }
}

void PT2257_write()
{
  Wire.beginTransmission(PT2257_Adr);
  delay(1);
  Wire.write(VOLH);
  delay(1);
  Wire.write(VOLL);
  delay(1);
  Wire.endTransmission();
}

void loop()
{
  ir_receive();
  key_read();
  ee_store();
  delay(SWDelay);
}
